import { String, ID, Number, Int, Float, Boolean, Unknown, Any, Json, ObjectType, Union, Intersect, Struct, Record, List, Tuple, Literal, Nullable, Strict, NonStrict, ReadOnly, ReadOnlyDeep, } from 'farrow-schema';
export const controvertTypes = (input) => {
    const controvertType = (input) => {
        switch (input.type) {
            case 'Scalar': {
                return controvertScalarType(input);
            }
            case 'Object': {
                return controvertObjectType(input);
            }
            case 'Union': {
                return controvertUnionType(input);
            }
            case 'Intersect': {
                return controvertIntersectType(input);
            }
            case 'Struct': {
                return controvertStructType(input);
            }
            case 'Record': {
                return controvertRecordType(input);
            }
            case 'List': {
                return controvertListType(input);
            }
            case 'Tuple': {
                return controvertTupleType(input);
            }
            case 'Literal': {
                return controvertLiteralType(input);
            }
            case 'Nullable': {
                return controvertNullableType(input);
            }
            case 'Strict': {
                return controvertStrictType(input);
            }
            case 'NonStrict': {
                return controvertNonStrictType(input);
            }
            case 'ReadOnly': {
                return controvertReadOnlyType(input);
            }
            case 'ReadOnlyDeep': {
                return controvertReadOnlyDeepType(input);
            }
        }
    };
    const controvertScalarType = (input) => {
        switch (input.valueName) {
            case 'String':
                return String;
            case 'ID':
                return ID;
            case 'Number':
                return Number;
            case 'Int':
                return Int;
            case 'Float':
                return Float;
            case 'Boolean':
                return Boolean;
            case 'Date':
                return Date;
            case 'Unknown':
                return Unknown;
            case 'Any':
                return Any;
            case 'Json':
                return Json;
            default:
                throw new Error(`Unknown Scalar Type name: ${input.valueName}`);
        }
    };
    const controvertObjectType = (input) => {
        const fields = controvertFieldsType(input.fields);
        return class Obj extends ObjectType {
            constructor() {
                super();
                for (const name in fields) {
                    // @ts-ignore
                    this[name] = fields[name];
                }
            }
        };
    };
    const controvertFieldsType = (input) => {
        const fields = {};
        for (const name in input) {
            fields[name] = controvertFieldType(input[name]);
        }
        return fields;
    };
    const controvertFieldType = (input) => {
        return {
            __type: findType(input.typeId),
            description: input.description,
            deprecated: input.deprecated,
        };
    };
    const controvertUnionType = (input) => {
        const items = input.itemTypes.map(({ typeId }) => findType(typeId));
        return Union(...items);
    };
    const controvertIntersectType = (input) => {
        const items = input.itemTypes.map(({ typeId }) => findType(typeId));
        return Intersect(...items);
    };
    const controvertStructType = (input) => {
        const fields = controvertFieldsType(input.fields);
        return Struct(fields);
    };
    const controvertRecordType = (input) => {
        const item = findType(input.valueTypeId);
        return Record(item);
    };
    const controvertListType = (input) => {
        const item = findType(input.itemTypeId);
        return List(item);
    };
    const controvertTupleType = (input) => {
        const items = input.itemTypes.map(({ typeId }) => findType(typeId));
        return Tuple(...items);
    };
    const controvertLiteralType = (input) => {
        return Literal(input.value);
    };
    const controvertNullableType = (input) => {
        const item = findType(input.itemTypeId);
        return Nullable(item);
    };
    const controvertStrictType = (input) => {
        const item = findType(input.itemTypeId);
        return Strict(item);
    };
    const controvertNonStrictType = (input) => {
        const item = findType(input.itemTypeId);
        return NonStrict(item);
    };
    const controvertReadOnlyType = (input) => {
        const item = findType(input.itemTypeId);
        return ReadOnly(item);
    };
    const controvertReadOnlyDeepType = (input) => {
        const item = findType(input.itemTypeId);
        return ReadOnlyDeep(item);
    };
    const findType = (typeId) => {
        const schemaCtor = types.get(typeId.toString());
        if (!schemaCtor) {
            throw new Error(`Unknown typeId: ${typeId}`);
        }
        return schemaCtor;
    };
    const types = new Map();
    for (const id in input) {
        types.set(id, controvertType(input[id]));
    }
    return types;
};
//# sourceMappingURL=types.js.map